@@ -2,7 +2,7 @@
 #include <sys/time.h>
 #include <ctype.h>
 
-unsigned int get_current_time()
+static unsigned int get_current_time(void)
 {
 	struct timeval tv;
 	gettimeofday(&tv, NULL);
@@ -27,9 +27,9 @@ int main(int argc, char** argv)
 			for (i = 0; i < seq->rnum; i++)
 			{
 				ccv_root_comp_t* comp = (ccv_root_comp_t*)ccv_array_get(seq, i);
-				printf("%d %d %d %d %f %d\n", comp->rect.x, comp->rect.y, comp->rect.width, comp->rect.height, comp->confidence, comp->pnum);
+				printf("%d %d %d %d %f %d\n", comp->rect.x, comp->rect.y, comp->rect.width, comp->rect.height, comp->classification.confidence, comp->pnum);
 				for (j = 0; j < comp->pnum; j++)
-					printf("| %d %d %d %d %f\n", comp->part[j].rect.x, comp->part[j].rect.y, comp->part[j].rect.width, comp->part[j].rect.height, comp->part[j].confidence);
+					printf("| %d %d %d %d %f\n", comp->part[j].rect.x, comp->part[j].rect.y, comp->part[j].rect.width, comp->part[j].rect.height, comp->part[j].classification.confidence);
 			}
 			printf("total : %d in time %dms\n", seq->rnum, elapsed_time);
 			ccv_array_free(seq);
@@ -60,9 +60,9 @@ int main(int argc, char** argv)
 					for (i = 0; i < seq->rnum; i++)
 					{
 						ccv_root_comp_t* comp = (ccv_root_comp_t*)ccv_array_get(seq, i);
-						printf("%s %d %d %d %d %f %d\n", file, comp->rect.x, comp->rect.y, comp->rect.width, comp->rect.height, comp->confidence, comp->pnum);
+						printf("%s %d %d %d %d %f %d\n", file, comp->rect.x, comp->rect.y, comp->rect.width, comp->rect.height, comp->classification.confidence, comp->pnum);
 						for (j = 0; j < comp->pnum; j++)
-							printf("| %d %d %d %d %f\n", comp->part[j].rect.x, comp->part[j].rect.y, comp->part[j].rect.width, comp->part[j].rect.height, comp->part[j].confidence);
+							printf("| %d %d %d %d %f\n", comp->part[j].rect.x, comp->part[j].rect.y, comp->part[j].rect.width, comp->part[j].rect.height, comp->part[j].classification.confidence);
 					}
 					ccv_array_free(seq);
 				}
