@@ -223,9 +223,9 @@ void ccv_canny(ccv_dense_matrix_t* a, ccv_dense_matrix_t** b, int type, int size
 		dxi += a->cols;
 		dyi += a->cols;
 		int* map = (int*)ccmalloc(sizeof(int) * (a->rows + 2) * (a->cols + 2));
-		memset(map, 0, sizeof(int) * (a->cols + 2));
-		int* map_ptr = map + a->cols + 2 + 1;
 		int map_cols = a->cols + 2;
+		memset(map, 0, sizeof(int) * map_cols);
+		int* map_ptr = map + map_cols + 1;
 		int** stack = (int**)ccmalloc(sizeof(int*) * a->rows * a->cols);
 		int** stack_top = stack;
 		int** stack_bottom = stack;
@@ -303,7 +303,7 @@ void ccv_canny(ccv_dense_matrix_t* a, ccv_dense_matrix_t** b, int type, int size
 			rows[1] = rows[2];
 			rows[2] = row;
 		}
-		memset(map_ptr - map_cols - 1, 0, sizeof(int) * (a->cols + 2));
+		memset(map_ptr - 1, 0, sizeof(int) * map_cols);
 		int dr[] = {-1, 1, -map_cols - 1, -map_cols, -map_cols + 1, map_cols - 1, map_cols, map_cols + 1};
 		while (stack_top > stack_bottom)
 		{
