@@ -222,7 +222,7 @@ void ccv_swt(ccv_dense_matrix_t* a, ccv_dense_matrix_t** b, int type, ccv_swt_pa
 	ccv_matrix_free(dy);
 }
 
-ccv_array_t* _ccv_swt_connected_component(ccv_dense_matrix_t* a, int ratio, int min_height, int max_height, int min_area)
+static ccv_array_t* _ccv_swt_connected_component(ccv_dense_matrix_t* a, int ratio, int min_height, int max_height, int min_area)
 {
 	int i, j, k;
 	int* a_ptr = a->data.i32;
@@ -538,7 +538,7 @@ static ccv_array_t* _ccv_swt_merge_textline(ccv_array_t* letters, ccv_swt_param_
 }
 
 #define less_than(a, b, aux) ((a)->center.x < (b)->center.x)
-CCV_IMPLEMENT_QSORT(_ccv_sort_letters, ccv_letter_t*, less_than)
+static CCV_IMPLEMENT_QSORT(_ccv_sort_letters, ccv_letter_t*, less_than)
 #undef less_than
 
 static ccv_array_t* _ccv_swt_break_words(ccv_array_t* textline, ccv_swt_param_t params)
@@ -550,6 +550,7 @@ static ccv_array_t* _ccv_swt_break_words(ccv_array_t* textline, ccv_swt_param_t
 		if (t->neighbors - 1 > n)
 			n = t->neighbors - 1;
 	}
+	assert(n > 0);
 	int* buffer = (int*)alloca(n * sizeof(int));
 	ccv_array_t* words = ccv_array_new(sizeof(ccv_rect_t), textline->rnum, 0);
 	for (i = 0; i < textline->rnum; i++)
@@ -657,7 +658,7 @@ ccv_array_t* ccv_swt_detect_words(ccv_dense_matrix_t* a, ccv_swt_param_t params)
 		ccv_array_t* idx = 0;
 		int ntl = ccv_array_group(textline, &idx, _ccv_is_same_textline, params.same_word_thresh);
 		ccv_array_t* words;
-		if (params.breakdown)
+		if (params.breakdown && ntl > 0)
 		{
 			textline2 = ccv_array_new(sizeof(ccv_textline_t), ntl, 0);
 			ccv_array_zero(textline2);
@@ -721,6 +722,7 @@ ccv_array_t* ccv_swt_detect_words(ccv_dense_matrix_t* a, ccv_swt_param_t params)
 	}
 	if (params.scale_invariant && params.min_neighbors)
 	{
+		assert(all_words);
 		// de-dup logic, similar to what BBF / DPM have
 		ccv_array_t* idx = 0;
 		int ntl = ccv_array_group(all_words, &idx, _ccv_is_same_textline, params.same_word_thresh);
