@@ -7,16 +7,26 @@
 static void uri_dpm_on_model_string(void* context, char* string);
 static void uri_dpm_on_source_blob(void* context, ebb_buf data);
 
+typedef struct {
+	ccv_dpm_param_t params;
+	int max_dimension;
+} ccv_dpm_uri_param_t;
+
 static const param_dispatch_t param_map[] = {
 	{
 		.property = "interval",
 		.type = PARAM_TYPE_INT,
-		.offset = offsetof(ccv_dpm_param_t, interval),
+		.offset = offsetof(ccv_dpm_uri_param_t, params) + offsetof(ccv_dpm_param_t, interval),
+	},
+	{
+		.property = "max_dimension",
+		.type = PARAM_TYPE_INT,
+		.offset = offsetof(ccv_dpm_uri_param_t, max_dimension),
 	},
 	{
 		.property = "min_neighbors",
 		.type = PARAM_TYPE_INT,
-		.offset = offsetof(ccv_dpm_param_t, min_neighbors),
+		.offset = offsetof(ccv_dpm_uri_param_t, params) + offsetof(ccv_dpm_param_t, min_neighbors),
 	},
 	{
 		.property = "model",
@@ -33,7 +43,7 @@ static const param_dispatch_t param_map[] = {
 	{
 		.property = "threshold",
 		.type = PARAM_TYPE_FLOAT,
-		.offset = offsetof(ccv_dpm_param_t, threshold),
+		.offset = offsetof(ccv_dpm_uri_param_t, params) + offsetof(ccv_dpm_param_t, threshold),
 	},
 };
 
@@ -46,7 +56,7 @@ typedef struct {
 typedef struct {
 	param_parser_t param_parser;
 	dpm_context_t* context;
-	ccv_dpm_param_t params;
+	ccv_dpm_uri_param_t params;
 	ccv_dpm_mixture_model_t* mixture_model;
 	ebb_buf source;
 } dpm_param_parser_t;
@@ -54,7 +64,8 @@ typedef struct {
 static void uri_dpm_param_parser_init(dpm_param_parser_t* parser)
 {
 	param_parser_init(&parser->param_parser, param_map, sizeof(param_map) / sizeof(param_dispatch_t), &parser->params, parser);
-	parser->params = ccv_dpm_default_params;
+	parser->params.params = ccv_dpm_default_params;
+	parser->params.max_dimension = 0;
 	parser->mixture_model = 0;
 	parser->source.data = 0;
 }
@@ -107,16 +118,16 @@ void* uri_dpm_detect_objects_init(void)
 	assert(param_parser_map_alphabet(param_map, sizeof(param_map) / sizeof(param_dispatch_t)) == 0);
 	context->desc = param_parser_map_http_body(param_map, sizeof(param_map) / sizeof(param_dispatch_t),
 		"[{"
-			"\"x\":\"integer\","
-			"\"y\":\"integer\","
-			"\"width\":\"integer\","
-			"\"height\":\"integer\","
+			"\"x\":\"number\","
+			"\"y\":\"number\","
+			"\"width\":\"number\","
+			"\"height\":\"number\","
 			"\"confidence\":\"number\","
 			"\"parts\":[{"
-				"\"x\":\"integer\","
-				"\"y\":\"integer\","
-				"\"width\":\"integer\","
-				"\"height\":\"integer\","
+				"\"x\":\"number\","
+				"\"y\":\"number\","
+				"\"width\":\"number\","
+				"\"height\":\"number\","
 				"\"confidence\":\"number\""
 			"}]"
 		"}]");
@@ -165,8 +176,16 @@ int uri_dpm_detect_objects(const void* context, const void* parsed, ebb_buf* buf
 		free(parser);
 		return -1;
 	}
-	ccv_array_t* seq = ccv_dpm_detect_objects(image, &parser->mixture_model, 1, parser->params);
-	ccv_matrix_free(image);
+	ccv_dense_matrix_t* resize = 0;
+	if (parser->params.max_dimension > 0 && (image->rows > parser->params.max_dimension || image->cols > parser->params.max_dimension))
+	{
+		ccv_resample(image, &resize, 0, ccv_min(parser->params.max_dimension, (int)(image->rows * (float)parser->params.max_dimension / image->cols + 0.5)), ccv_min(parser->params.max_dimension, (int)(image->cols * (float)parser->params.max_dimension / image->rows + 0.5)), CCV_INTER_AREA);
+		ccv_matrix_free(image);
+	} else
+		resize = image;
+	ccv_array_t* seq = ccv_dpm_detect_objects(resize, &parser->mixture_model, 1, parser->params.params);
+	float width = resize->cols, height = resize->rows;
+	ccv_matrix_free(resize);
 	if (seq  == 0)
 	{
 		free(parser);
@@ -183,7 +202,7 @@ int uri_dpm_detect_objects(const void* context, const void* parsed, ebb_buf* buf
 		{
 			char cell[128];
 			ccv_root_comp_t* comp = (ccv_root_comp_t*)ccv_array_get(seq, i);
-			snprintf(cell, 128, "{\"x\":%d,\"y\":%d,\"width\":%d,\"height\":%d,\"confidence\":%f,\"parts\":[", comp->rect.x, comp->rect.y, comp->rect.width, comp->rect.height, comp->confidence);
+			snprintf(cell, 128, "{\"x\":%f,\"y\":%f,\"width\":%f,\"height\":%f,\"confidence\":%f,\"parts\":[", comp->rect.x / width, comp->rect.y / height, comp->rect.width / width, comp->rect.height / height, comp->classification.confidence);
 			size_t len = strnlen(cell, 128);
 			while (buf->written + len >= buf->len)
 			{
@@ -194,7 +213,7 @@ int uri_dpm_detect_objects(const void* context, const void* parsed, ebb_buf* buf
 			buf->written += len;
 			for (j = 0; j < comp->pnum; j++)
 			{
-				snprintf(cell, 128, "{\"x\":%d,\"y\":%d,\"width\":%d,\"height\":%d,\"confidence\":%f}", comp->part[j].rect.x, comp->part[j].rect.y, comp->part[j].rect.width, comp->part[j].rect.height, comp->part[j].confidence);
+				snprintf(cell, 128, "{\"x\":%f,\"y\":%f,\"width\":%f,\"height\":%f,\"confidence\":%f}", comp->part[j].rect.x / width, comp->part[j].rect.y / height, comp->part[j].rect.width / width, comp->part[j].rect.height / height, comp->part[j].classification.confidence);
 				len = strnlen(cell, 128);
 				while (buf->written + len + 3 >= buf->len)
 				{
@@ -210,7 +229,7 @@ int uri_dpm_detect_objects(const void* context, const void* parsed, ebb_buf* buf
 			data[buf->written - 1] = (i == seq->rnum - 1) ? ']' : ',';
 		}
 		char http_header[192];
-		snprintf(http_header, 192, ebb_http_header, buf->written);
+		snprintf(http_header, 192, ebb_http_header, buf->written + 1);
 		size_t len = strnlen(http_header, 192);
 		if (buf->written + len + 1 >= buf->len)
 		{
